#!/bin/python
# Vietoris-Rips complex visualization of ceiling fan sonar echos
#
# Copyright (c) 2015, Michael Robinson
# Distribution of unaltered copies permitted for noncommercial use only
# All other uses require express permission of the author
# This software comes with no warrantees express or implied

import os
import numpy as np
import matplotlib.pyplot as plt
import simplicialHomology as sh

def plot_complex(locations,complex,color=None):
    """Plot a complex given locations of nodes"""
    colors='cbgr'
    for idx,f in enumerate(complex):
        if color != None:
            col=colors[color[idx]]
        else:
            col='g'
        if len(f)>2:
            for face in sh.ksublists(f,3):
                x,y=zip(*[locations[i] for i in face])
                plt.fill(x,y,col,alpha=0.5)
        elif len(f)>=2:
            x,y=zip(*[locations[i] for i in f])
            plt.fill(x,y,col,alpha=0.5)

    x,y=zip(*locations)
    plt.plot(x,y,'k.',ms=4*np.pi)
    for i in range(len(locations)):
        plt.text(x[i],y[i]+0.001,str(i))
    return

# Ingest data
ptsmat=np.loadtxt('collection_3.txt')
ptsmat=ptsmat[250:,1:]

# Run PCA for visualization
u,s,v=np.linalg.svd(np.dot(ptsmat,ptsmat.T))
locations=np.dot(v[1:3,:],ptsmat).T

# Construct Vietoris Rips complex
# VARY THE DIAMETER PARAMETER
vr=sh.vietorisRips(ptsmat.T,diameter=0.018)

plot_complex(locations,vr)
plt.savefig('vietoris_rips.png')
plt.show()

