# Construction of transmission line sheaves for geometry sounding
#
# Copyright (c) 2015, Michael Robinson
# Distribution of unaltered copies permitted for noncommercial use only
# All other uses require express permission of the author
# This software comes with no warrantees express or implied

import pysheaf as ps
import numpy as np
import networkx as nx
import matplotlib.pyplot as plt

# Simulated geometry
ft2m=12*0.0254 # Conversion feet to meters
dg=ps.DirectedGraph([(None,1),(None,2),(None,3),(1,2),(2,3),(3,1)])
dg.cells[3].length=70*ft2m # Edge e3
dg.cells[4].length=(150+70)*ft2m # Edge e1
dg.cells[5].length=150*ft2m # Edge e2

frequencies=[905e6,2451e6] # Operating frequencies
wavenumbers=[2*np.pi*f/3e8 for f in frequencies]

sheaves=[ps.TransLineSheaf(dg,k) for k in wavenumbers]

nx.draw(dg.attachDiagram())
plt.savefig('s3a1.png')
plt.show()
