# Switching sheaf morphism example
#
# Copyright (c) 2015, Michael Robinson
# Distribution of unaltered copies permitted for noncommercial use only
# All other uses require express permission of the author
# This software comes with no warrantees express or implied

import numpy as np
import pysheaf as ps
import networkx as nx
import matplotlib.pyplot as plt

# Connecting logic function
fcns=[(A,B,C,D) for A in [0,1] 
      for B in [0,1]
      for C in [0,1]
      for D in [0,1]]

for (A,B,C,D) in fcns:
    
    sh1=ps.Sheaf([ps.SheafCell(dimension=0,compactClosure=True,cofaces=[ps.SheafCoface(index=1,orientation=1,corestriction=np.matrix([[1,1,1,1,0,0,0,0],[0,0,0,0,1,1,1,1]])),ps.SheafCoface(index=2,orientation=1,corestriction=np.matrix([[1,1,0,0,1,1,0,0],[0,0,1,1,0,0,1,1]])),ps.SheafCoface(index=3,orientation=1,corestriction=np.matrix([[1,0,1,0,0,1,0,1],[0,1,0,1,1,0,1,0]]))]),
                  ps.SheafCell(dimension=1,compactClosure=False,stalkDim=2,cofaces=[]),
                  ps.SheafCell(dimension=2,compactClosure=False,stalkDim=2,cofaces=[]),
                  ps.SheafCell(dimension=3,compactClosure=False,stalkDim=2,cofaces=[])])
    
    sh2=ps.Sheaf([ps.SheafCell(dimension=0,compactClosure=True,cofaces=[ps.SheafCoface(index=1,orientation=1,corestriction=np.matrix([[1,1,0,0],[0,0,1,1]])),ps.SheafCoface(index=2,orientation=1,corestriction=np.matrix([[1,0,1,0],[0,1,0,1]])),ps.SheafCoface(index=4,orientation=-1,corestriction=np.matrix([[1-A,1-B,1-C,1-D],[A,B,C,D]]))]),
                  ps.SheafCell(dimension=1,compactClosure=False,stalkDim=2,cofaces=[]),
                  ps.SheafCell(dimension=1,compactClosure=False,stalkDim=2,cofaces=[]),
                  ps.SheafCell(dimension=0,compactClosure=True,cofaces=[ps.SheafCoface(index=4,orientation=1,corestriction=np.matrix([[1,1,0,0],[0,0,1,1]])),ps.SheafCoface(index=5,orientation=1,corestriction=np.matrix([[1,0,1,0],[0,1,0,1]]))]),
                  ps.SheafCell(dimension=1,compactClosure=True,stalkDim=2,cofaces=[]),
                  ps.SheafCell(dimension=1,compactClosure=False,stalkDim=2,cofaces=[])])

    mor=[ps.SheafMorphismCell(destinations=[3,4,0],maps=[np.matrix([[1-A,0,1-B,0,1-C,0,1-D,0],
                                                                [A,0,B,0,C,0,D,0],
                                                                [0,1-A,0,1-B,0,1-C,0,1-D],
                                                                    [0,A,0,B,0,C,0,D]]),
                                                         np.matrix([[1-A,1-A,1-B,1-B,1-C,1-C,1-D,1-D],[A,A,B,B,C,C,D,D]]),
                                                         np.matrix([[1,1,0,0,0,0,0,0],[0,0,1,1,0,0,0,0],[0,0,0,0,1,1,0,0],[0,0,0,0,0,0,1,1]])]),
         ps.SheafMorphismCell(destinations=[1],maps=[np.eye(2)]),
         ps.SheafMorphismCell(destinations=[2],maps=[np.eye(2)]),
         ps.SheafMorphismCell(destinations=[5],maps=[np.eye(2)])]
    
    ind=ps.inducedMap(sh1,sh2,mor,0)

    if ps.matrixrank(ind) != 6:
        print '--------------------------------------------'
        print 'First gate logic function: ' + str((A,B,C,D))
        print 'Source 0th Betti: ' + str(sh1.betti(0))
        print 'Target 0th Betti: ' + str(sh2.betti(0))
        print 'Induced map rank: ' + str(ps.matrixrank(ind))
        print 'Induced map on 0th cohomology:\n' + str(ind)
        print 'Cokernel:\n' + str(ps.cokernel(ind))

plt.figure(1)
plt.subplot(121)
nx.draw(sh1.attachDiagram())
plt.subplot(122)
nx.draw(sh2.attachDiagram())
plt.savefig('s4a4.svg')
plt.show()
