# Measure local homology of protein-protein networks
#
# Copyright (c) 2015, Michael Robinson
# Distribution of unaltered copies permitted for noncommercial use only
# All other uses require express permission of the author
# This software comes with no warrantees express or implied

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import networkx as nx
import simplicialHomology as sh

# Betti numbers to examine
ks = range(8)

# Number of Proteins to try
prots = 1500

# Load the data
dfPP=pd.read_table('baylor_ppi.txt',sep='\t* *')

dims=[]
# Iterate over some entities
for i,e1 in enumerate(dfPP['entity1']):
   if i > prots: break

   # Compute the star over this entity
   verts=dfPP['entity1'][dfPP['entity2'][:]==e1]
   verts.append(dfPP['entity2'][dfPP['entity1'][:]==e1])
   
   # Filter out edges that connect within the star
   subgraph=dfPP[['entity1','entity2']][dfPP['entity1'].isin(verts) & dfPP['entity2'].isin(verts)]

   if len(subgraph) > 0:  
      # Use NetworkX to build a toplex model of the vicinity of the entity
      G=nx.Graph()
      G.add_edges_from(subgraph.values)
      toplexes = [[e1] + clique for clique in nx.find_cliques(G)]
      
      # Find the largest clique adjacent to the entity
      tplx=max(toplexes,key=len)

      dims.append([sh.localHomology(toplexes,k,tplx).shape[1] for k in ks])

np.savetxt('pp_localbetti.txt',np.array(dims),delimiter=',',fmt='%d')

plt.figure(1)
plt.hist(np.array(dims),bins=8,label=['Degree ' + str(k) for k in ks],histtype='barstacked')
plt.title('Local Betti number distribution')
plt.xlabel('Dimension of local homology space')
plt.ylabel('Frequency')
plt.legend()
plt.savefig('s4a2_localbetti.png')
plt.show()

